@@ -5,10 +5,14 @@ namespace FSpot {
 			try {
 				scroll = new Gtk.ScrolledWindow (null, null);
 				view = new PhotoImageView (query);
+				view.ModifyBg (Gtk.StateType.Normal, this.Style.Black);
+
 				this.Add (scroll);
 				scroll.Add (view);
 				scroll.ShowAll ();
+				this.Decorated = false;
 				this.Fullscreen ();
+				this.ButtonPressEvent += HandleButtonPressEvent;
 			} catch (System.Exception e) {
 				System.Console.WriteLine (e);
 			}		      
@@ -25,6 +29,15 @@ namespace FSpot {
 		private Gtk.Button forward_button;
 		private Gtk.Button back_button;
 		
+		private void HandleButtonPressEvent (object sender, Gtk.ButtonPressEventArgs args)
+		{
+			if (args.Event.Type == Gdk.EventType.ButtonPress
+			    && args.Event.Button == 3) {
+				PhotoPopup popup = new PhotoPopup ();
+				popup.Activate (args.Event);
+			}
+		}
+
 		protected override bool OnKeyPressEvent (Gdk.EventKey key)
 		{
 			if (key == null) {
@@ -37,28 +50,12 @@ namespace FSpot {
 				return false;
 			}
 
-			switch (key.Key) {
-			case Gdk.Key.Page_Up:
-			case Gdk.Key.KP_Page_Up:
-				view.Prev ();
-				break;
-			case Gdk.Key.Page_Down:
-			case Gdk.Key.KP_Page_Down:
-				view.Next ();
-				break;
-			case Gdk.Key.F:
-			case Gdk.Key.f:
-				view.Fit = true;
-				break;
-			default:
-				bool retval = base.OnKeyPressEvent (key);
-				if (!retval)
-					this.Destroy ();
-				else 
-					view.Fit = false;
-				return retval;
-			}
-			return true;
+			bool retval = base.OnKeyPressEvent (key);
+			if (!retval)
+				this.Destroy ();
+			else 
+				view.Fit = false;
+			return retval;
 		}
 	}
 }
