@@ -8,6 +8,69 @@ using System.IO;
 using System;
 
 public class ImportCommand : FSpot.GladeDialog {
+	private class VolumeItem : Gtk.ImageMenuItem {
+		public Gnome.Vfs.Volume Volume;
+		
+		public VolumeItem (Gnome.Vfs.Volume vol) : base (vol.DisplayName)
+		{
+			this.Volume = vol;
+			this.Image = new Gtk.Image (vol.Icon);
+
+			Gdk.Pixbuf icon = PixbufUtils.LoadThemeIcon (vol.Icon, 32);
+			if (icon != null)
+				this.Image = new Gtk.Image (icon);
+
+			
+		}
+
+	}
+
+	private class DriveItem : Gtk.ImageMenuItem {
+		public Gnome.Vfs.Drive Drive;
+		
+		public DriveItem (Gnome.Vfs.Drive drive) : base (drive.DisplayName)
+		{
+			this.Drive = drive;
+
+			Gdk.Pixbuf icon;
+			if (drive.IsMounted) {
+				icon = PixbufUtils.LoadThemeIcon (drive.MountedVolume.Icon, 32);
+				this.Sensitive = drive.MountedVolume.IsMounted;
+			} else {
+				icon = PixbufUtils.LoadThemeIcon (drive.Icon, 32);
+			}
+
+			if (icon != null)
+				this.Image = new Gtk.Image (icon);
+		}
+	}
+	
+	private class SourceMenu : Gtk.Menu {
+		public SourceMenu () {
+			Gnome.Vfs.VolumeMonitor monitor = Gnome.Vfs.VolumeMonitor.Get ();
+
+			foreach (Gnome.Vfs.Drive drive in monitor.ConnectedDrives) {
+				 this.Append (new DriveItem (drive));
+			 }
+
+			 foreach (Gnome.Vfs.Volume vol in monitor.MountedVolumes) {
+				 System.Console.WriteLine ("{0} - {1} - {2} {3} {4}",
+							   vol.DisplayName, 
+							   vol.Icon, 
+							   vol.VolumeType.ToString (), 
+							   vol.ActivationUri, 
+							   vol.IsMounted);
+
+				 if (vol.Drive != null)
+					 System.Console.WriteLine (vol.Drive.DeviceType.ToString ());
+								   
+				if (vol.IsUserVisible)
+					this.Append (new VolumeItem (vol));
+			}
+
+			this.ShowAll ();
+		}
+	}
 
 	private class PhotoGrid : Table {
 		const int NUM_COLUMNS = 5;
@@ -20,7 +83,7 @@ public class ImportCommand : FSpot.GladeDialog {
 
 		Gtk.Image [] image_widgets;
 
-		int position;
+		int position = -1;
 		
 		/*
 		 * This controls whether or wraps as new photos are added.
@@ -94,6 +157,8 @@ public class ImportCommand : FSpot.GladeDialog {
 
 	[Glade.Widget] Gtk.Entry import_folder_entry;
 	[Glade.Widget] Gtk.OptionMenu tag_option_menu;
+	[Glade.Widget] Gtk.CheckButton attach_check;
+	[Glade.Widget] Gtk.CheckButton recurse_check;
 	[Glade.Widget] Gtk.Image tag_image;
 	[Glade.Widget] Gtk.Label tag_label;
 	
@@ -136,7 +201,7 @@ public class ImportCommand : FSpot.GladeDialog {
 	private void UpdateProgressBar (int count, int total)
 	{
 		progress_bar.Text = String.Format ("Importing {0} of {1}", count, total);
-		progress_bar.Fraction = (double) count / total;
+		progress_bar.Fraction = (double) count / System.Math.Max (total, 1);
 	}
 
 	private int DoImport (FileImportBackend importer)
@@ -148,7 +213,7 @@ public class ImportCommand : FSpot.GladeDialog {
 
 		cancelled = false;
 		bool ongoing = true;
-		while (ongoing) {
+		while (ongoing && total > 0) {
 			Photo photo;
 			Pixbuf thumbnail;
 			int count;
@@ -165,7 +230,7 @@ public class ImportCommand : FSpot.GladeDialog {
 				Console.WriteLine ("Could not import file");
 				continue;
 			}
-			
+
 			grid.AddThumbnail (thumbnail);
 			UpdateProgressBar (count, total);
 		}
@@ -192,15 +257,21 @@ public class ImportCommand : FSpot.GladeDialog {
 		}
 	}
 	
-	public void HandleImportBrowse (object o, EventArgs args) {
+	public void HandleTagToggled (object o, EventArgs args) 
+	{
+		tag_option_menu.Sensitive = attach_check.Active;
+	}
+
+	public void HandleImportBrowse (object o, EventArgs args) 
+	{
 	
 		CompatFileChooserDialog file_selector =
-			new CompatFileChooserDialog ("Import", main_window,
+			new CompatFileChooserDialog ("Import", this.Dialog,
 						     CompatFileChooserDialog.Action.SelectFolder);
-		file_selector.SelectMultiple = true;
 
-		
+		file_selector.SelectMultiple = false;
 		file_selector.Filename = import_folder_entry.Text;
+
 		int response = file_selector.Run ();
 
 		if ((ResponseType) response == ResponseType.Ok) {
@@ -212,30 +283,48 @@ public class ImportCommand : FSpot.GladeDialog {
 		
 	}
 	
-	public void HandleTagMenuSelected (Tag t) {
+	public void HandleTagMenuSelected (Tag t) 
+	{
 		tag_selected = t;
 		//tag_image.Pixbuf = t.Icon;
 		//tag_label.Text = t.Name;
 	
 	}
 	
+	public void HandleEntryActivate (object sender, EventArgs args)
+	{
+		this.Dialog.Respond (Gtk.ResponseType.Ok);
+	}
+
 	public int ImportFromFile (PhotoStore store, string path)
 	{
 		this.CreateDialog ("import_dialog");
 		
+		this.Dialog.TransientFor = main_window;
+		this.Dialog.WindowPosition = Gtk.WindowPosition.CenterOnParent;
+
 		//Gtk.Menu menu = new Gtk.Menu();
 		MenuItem attach_item = new MenuItem (Mono.Posix.Catalog.GetString ("Select Tag"));
 		TagMenu tagmenu = new TagMenu (null, MainWindow.Toplevel.Database.Tags);
 		
+		this.Dialog.DefaultResponse = ResponseType.Ok;
+		
+		import_folder_entry.Activated += HandleEntryActivate;
+
 		tagmenu.TagSelected += HandleTagMenuSelected;
 		tagmenu.ShowAll ();
 		tagmenu.Populate (true);
 		tagmenu.Prepend (attach_item);
 		
 		tag_option_menu.Menu = tagmenu;
-		
+		//tag_option_menu.Menu = new SourceMenu ();
+
 		tag_selected = null;
-				
+		if (attach_check != null) {
+			attach_check.Toggled += HandleTagToggled;
+			HandleTagToggled (null, null);
+		}				
+
 		if (path != null)
 			import_folder_entry.Text = path;
 		else 
@@ -243,11 +332,35 @@ public class ImportCommand : FSpot.GladeDialog {
 						
 		ResponseType response = (ResponseType) this.Dialog.Run ();
 		
+		while (response == ResponseType.Ok) {
+			if (System.IO.Directory.Exists (import_folder_entry.Text))
+			    break;
+
+			HigMessageDialog md = new HigMessageDialog (this.Dialog,
+								    DialogFlags.DestroyWithParent,
+								    MessageType.Error,
+								    ButtonsType.Ok,
+								    Mono.Posix.Catalog.GetString ("Directory does not exist."),
+									    String.Format (Mono.Posix.Catalog.GetString ("The directory you selected \"{0}\" does not exist.  Please choose a different directory"), import_folder_entry.Text));
+			md.Run ();
+			md.Destroy ();
+
+			response = (Gtk.ResponseType) this.Dialog.Run ();
+		}
+
 		if (response == ResponseType.Ok) {
 			string [] pathimport =  {import_folder_entry.Text};
 			this.Dialog.Destroy();
+
+			Tag [] tags = null;		       
+			if (attach_check.Active && tag_selected != null)
+				tags = new Tag [] {tag_selected};
 			
-			return DoImport (new FileImportBackend (store, pathimport, true, tag_selected != null ? new Tag [] {tag_selected} : null));
+			bool recurse = true;
+			if (recurse_check != null)
+				recurse = recurse_check.Active;
+
+			return DoImport (new FileImportBackend (store, pathimport, recurse, tags));
 				
 		} else {
 			this.Dialog.Destroy();
@@ -257,7 +370,7 @@ public class ImportCommand : FSpot.GladeDialog {
 
 	public int ImportFromPaths (PhotoStore store, string [] paths)
 	{
-		return ImportFromPaths (store, paths);
+		return ImportFromPaths (store, paths, null);
 	}
 
 	public int ImportFromPaths (PhotoStore store, string [] paths, Tag [] tags)
