@@ -10,15 +10,12 @@ using System.Collections;
 using System.Runtime.InteropServices;
 using System.Text.RegularExpressions;
 
+using LibGPhoto2;
+
 public class MainWindow {
         public static MainWindow Toplevel = null;
 
 	Db db;
-	public Db Database {
-		get {
-			return db;
-		}
-	}
 
 	TagSelectionWidget tag_selection_widget;
 	[Glade.Widget] Gtk.Window main_window;
@@ -62,6 +59,8 @@ public class MainWindow {
 	[Glade.Widget] MenuItem remove_tag;
 	[Glade.Widget] MenuItem find_tag;
 
+	[Glade.Widget] VPaned info_vpaned;
+
 	PhotoVersionMenu versions_submenu;
 
 	Gtk.ToggleButton browse_button;
@@ -78,7 +77,8 @@ public class MainWindow {
 	FSpot.Delay slide_delay;
 	
 	string last_import_path;
-	
+	ModeType view_mode;
+
 	// Drag and Drop
 	enum TargetType {
 		UriList,
@@ -108,39 +108,6 @@ public class MainWindow {
 	};
 
 	const int PHOTO_IDX_NONE = -1;
-	
-	struct YearCount {
-		int Year;
-		int Count;
-	}
-
-	// Index into the PhotoQuery.  If -1, no photo is selected or multiple photos are selected.
-	private int ActiveIndex () {
-		if (view_mode == ModeType.IconView && icon_view.CurrentIdx != -1)
-			return icon_view.CurrentIdx;
-
-	        int [] selection = SelectedIds ();
-		if (selection.Length == 1) 
-			return selection [0];
-		else 
-			return PHOTO_IDX_NONE;
-	}
-
-	public bool PhotoSelectionActive ()
-	{
-		return SelectedIds().Length > 0;
-	}
-
-	private Photo CurrentPhoto {
-		get {
-			int active = ActiveIndex ();
-			if (active >= 0)
-				return query.Photos [active];
-			else
-				return null;
-		}
-	}
-
 
 	//
 	// Constructor
@@ -171,21 +138,22 @@ public class MainWindow {
 		tag_selection_widget = new TagSelectionWidget (db.Tags);
 		tag_selection_scrolled.Add (tag_selection_widget);
 		
-		tag_selection_widget.Selection.Changed += new EventHandler (HandleTagSelectionChanged);
-		tag_selection_widget.SelectionChanged += new TagSelectionWidget.SelectionChangedHandler (OnTagSelectionChanged);
-		tag_selection_widget.DragDataGet += new DragDataGetHandler (HandleTagSelectionDragDataGet);
+		tag_selection_widget.Selection.Changed += HandleTagSelectionChanged;
+		tag_selection_widget.SelectionChanged += OnTagSelectionChanged;
+		tag_selection_widget.DragDataGet += HandleTagSelectionDragDataGet;
+		tag_selection_widget.DragDrop += HandleTagSelectionDragDrop;
 		Gtk.Drag.SourceSet (tag_selection_widget, Gdk.ModifierType.Button1Mask | Gdk.ModifierType.Button3Mask,
 				    tag_target_table, DragAction.Copy | DragAction.Move);
 
-		tag_selection_widget.DragDataReceived += new DragDataReceivedHandler (HandleTagSelectionDragDataReceived);
-		tag_selection_widget.DragMotion += new DragMotionHandler (HandleTagSelectionDragMotion);
+		tag_selection_widget.DragDataReceived += HandleTagSelectionDragDataReceived;
+		tag_selection_widget.DragMotion += HandleTagSelectionDragMotion;
 		Gtk.Drag.DestSet (tag_selection_widget, DestDefaults.All, tag_dest_target_table, 
-				  DragAction.Copy); 
+				  DragAction.Copy | DragAction.Move ); 
 
-		tag_selection_widget.ButtonPressEvent += new ButtonPressEventHandler (HandleTagSelectionButtonPressEvent);
+		tag_selection_widget.ButtonPressEvent += HandleTagSelectionButtonPressEvent;
 
 		info_box = new InfoBox ();
-		info_box.VersionIdChanged += new InfoBox.VersionIdChangedHandler (HandleInfoBoxVersionIdChange);
+		info_box.VersionIdChanged += HandleInfoBoxVersionIdChange;
 		left_vbox.PackStart (info_box, false, true, 0);
 
 		query = new FSpot.PhotoQuery (db.Photos);
@@ -208,8 +176,8 @@ public class MainWindow {
 
 		icon_view = new QueryView (query);
 		icon_view_scrolled.Add (icon_view);
-		icon_view.SelectionChanged += new IconView.SelectionChangedHandler (HandleSelectionChanged);
-		icon_view.DoubleClicked += new IconView.DoubleClickedHandler (HandleDoubleClicked);
+		icon_view.SelectionChanged += HandleSelectionChanged;
+		icon_view.DoubleClicked += HandleDoubleClicked;
 		icon_view.GrabFocus ();
 
 		new FSpot.PreviewPopup (icon_view);
@@ -217,8 +185,8 @@ public class MainWindow {
 		Gtk.Drag.SourceSet (icon_view, Gdk.ModifierType.Button1Mask | Gdk.ModifierType.Button3Mask,
 				    icon_source_target_table, DragAction.Copy | DragAction.Move);
 		
-		icon_view.DragBegin += new DragBeginHandler (HandleIconViewDragBegin);
-		icon_view.DragDataGet += new DragDataGetHandler (HandleIconViewDragDataGet);
+		icon_view.DragBegin += HandleIconViewDragBegin;
+		icon_view.DragDataGet += HandleIconViewDragDataGet;
 
 		TagMenu menu = new TagMenu (attach_tag, db.Tags);
 		menu.TagSelected += HandleAttachTagMenuSelected;
@@ -234,9 +202,9 @@ public class MainWindow {
 				  DragAction.Copy | DragAction.Move); 
 
 		//		icon_view.DragLeave += new DragLeaveHandler (HandleIconViewDragLeave);
-		icon_view.DragMotion += new DragMotionHandler (HandleIconViewDragMotion);
-		icon_view.DragDrop += new DragDropHandler (HandleIconViewDragDrop);
-		icon_view.DragDataReceived += new DragDataReceivedHandler (HandleIconViewDragDataReceived);
+		icon_view.DragMotion += HandleIconViewDragMotion;
+		icon_view.DragDrop += HandleIconViewDragDrop;
+		icon_view.DragDataReceived += HandleIconViewDragDataReceived;
 
 		photo_view = new PhotoView (query, db.Photos);
 		photo_box.Add (photo_view);
@@ -249,11 +217,11 @@ public class MainWindow {
 		Gtk.Drag.DestSet (photo_view, DestDefaults.All, tag_target_table, 
 				  DragAction.Copy | DragAction.Move); 
 
-		photo_view.DragMotion += new DragMotionHandler (HandlePhotoViewDragMotion);
-		photo_view.DragDrop += new DragDropHandler (HandlePhotoViewDragDrop);
-		photo_view.DragDataReceived += new DragDataReceivedHandler (HandlePhotoViewDragDataReceived);
+		photo_view.DragMotion += HandlePhotoViewDragMotion;
+		photo_view.DragDrop += HandlePhotoViewDragDrop;
+		photo_view.DragDataReceived += HandlePhotoViewDragDataReceived;
 
-		view_notebook.SwitchPage += new SwitchPageHandler (HandleViewNotebookSwitchPage);
+		view_notebook.SwitchPage += HandleViewNotebookSwitchPage;
 		adaptor.GlassSet += HandleAdaptorGlassSet;
 
 		UpdateMenus ();
@@ -262,6 +230,51 @@ public class MainWindow {
 
 		if (Toplevel == null)
 			Toplevel = this;
+
+		UpdateToolbar ();
+
+		if (db.Empty)
+			HandleImportCommand (null, null);
+	}
+
+	// Index into the PhotoQuery.  If -1, no photo is selected or multiple photos are selected.
+	private int ActiveIndex () 
+	{
+		if (view_mode == ModeType.IconView && icon_view.CurrentIdx != -1)
+			return icon_view.CurrentIdx;
+
+	        int [] selection = SelectedIds ();
+		if (selection.Length == 1) 
+			return selection [0];
+		else 
+			return PHOTO_IDX_NONE;
+	}
+
+	public bool PhotoSelectionActive ()
+	{
+		return SelectedIds().Length > 0;
+	}
+
+	private Photo CurrentPhoto {
+		get {
+			int active = ActiveIndex ();
+			if (active >= 0)
+				return query.Photos [active];
+			else
+				return null;
+		}
+	}
+
+	public Db Database {
+		get {
+			return db;
+		}
+	}
+
+	public ModeType ViewMode {
+		get {
+			return view_mode;
+		}
 	}
 
 	// Switching mode.
@@ -270,14 +283,6 @@ public class MainWindow {
 		PhotoView
 	};
 
-	ModeType view_mode;
-
-	public ModeType ViewMode {
-		get {
-			return view_mode;
-		}
-	}
-
 	public void SetViewMode (ModeType value)
 	{
 		view_mode = value;
@@ -441,6 +446,25 @@ public class MainWindow {
 		}
 	}
 
+	void HandleTagSelectionDragDataGet (object sender, DragDataGetArgs args)
+	{		
+		UriList list = new UriList (SelectedPhotos ());
+
+		switch (args.Info) {
+		case (uint) TargetType.TagList:
+			Byte [] data = Encoding.UTF8.GetBytes (list.ToString ());
+			Atom [] targets = args.Context.Targets;
+		
+			args.SelectionData.Set (targets[0], 8, data, data.Length);
+			break;
+		} 
+	}
+
+	void HandleTagSelectionDragDrop (object sender, DragDropArgs args)
+	{
+		args.RetVal = true;
+	}
+
 	public void HandleTagSelectionDragMotion (object o, DragMotionArgs args)
 	{
 		TreePath path;
@@ -485,15 +509,6 @@ public class MainWindow {
 		}
 	}
 
-	void HandleTagSelectionDragDataGet (object sender, DragDataGetArgs args)
-	{		
-		UriList list = new UriList (SelectedPhotos ());
-		Byte [] data = Encoding.UTF8.GetBytes (list.ToString ());
-		Atom [] targets = args.Context.Targets;
-		
-		args.SelectionData.Set (targets[0], 8, data, data.Length);
-	}
-
 #if SHOW_CALENDAR
 	void HandleCalendarDaySelected (object sender, System.EventArgs args)
 	{
@@ -552,16 +567,22 @@ public class MainWindow {
 
 			bool use_icon = false;;
 			while (len-- > 0) {
-				string thumbnail_path = Thumbnail.PathForUri (photos[len].DefaultVersionUri.ToString (), ThumbnailSize.Large);
-				Pixbuf thumbnail = ThumbnailCache.Default.GetThumbnailForPath (thumbnail_path);
+				string thumbnail_path = FSpot.ThumbnailGenerator.ThumbnailPath (photos [len].DefaultVersionUri);
+				FSpot.PixbufCache.CacheEntry entry = icon_view.Cache.Lookup (thumbnail_path);
+
+				Pixbuf thumbnail = null;
+				if (entry != null)
+					thumbnail = entry.ShallowCopyPixbuf ();
+				
 				if (thumbnail != null) {
 					Pixbuf small = PixbufUtils.ScaleToMaxSize (thumbnail, size, size);				
 
 					int x = border + len * (size/2) + (size - small.Width)/2;
 					int y = border + len * (size/2) + (size - small.Height)/2;
-					Pixbuf box = new Pixbuf (container, x - border, y - border, small.Width + 2 * border, small.Height + 2 * border);
-					box.Fill (0x000000ff);
+					Pixbuf box = new Pixbuf (container, x - border, y - border, 
+								 small.Width + 2 * border, small.Height + 2 * border);
 
+					box.Fill (0x000000ff);
 					small.CopyArea (0, 0, small.Width, small.Height, container, x, y); 
 					
 					thumbnail.Dispose ();
@@ -577,20 +598,19 @@ public class MainWindow {
 
 	void HandleIconViewDragDataGet (object sender, DragDataGetArgs args)
 	{		
-		UriList list = new UriList (SelectedPhotos ());
-
 		switch (args.Info) {
-		case (uint) TargetType.TagList:
+		case (uint) TargetType.UriList:
+		case (uint) TargetType.PhotoList:
+			UriList list = new UriList (SelectedPhotos ());
 			Byte [] data = Encoding.UTF8.GetBytes (list.ToString ());
 			Atom [] targets = args.Context.Targets;
-		
 			args.SelectionData.Set (targets[0], 8, data, data.Length);
 			break;
 		case (uint) TargetType.RootWindow:
-			System.Console.WriteLine ("Got background drop");
 			HandleSetAsBackgroundCommand (null, null);
-			break;
-		} 
+                        break;
+		}
+		       
 	}
 
 	void HandleIconViewDragDrop (object sender, DragDropArgs args)
@@ -621,7 +641,7 @@ public class MainWindow {
 	void HandleIconViewDragDataReceived (object sender, DragDataReceivedArgs args)
 	{
 	 	Widget source = Gtk.Drag.GetSourceWidget (args.Context);     
-
+		
 		switch (args.Info) {
 		case (uint)TargetType.TagList:
 			//
@@ -790,13 +810,65 @@ public class MainWindow {
 
 	void HandleImportCommand (object sender, EventArgs e)
 	{
+		db.Sync = false;
 		ImportCommand command = new ImportCommand (main_window);
 		if (command.ImportFromFile (db.Photos, this.last_import_path) > 0) {
 			this.last_import_path = command.ImportPath;
 			UpdateQuery ();
 		}
+		db.Sync = true;		
 	}
 
+	void HandleImportFromCameraCommand (object sender, EventArgs e)
+	{
+		GPhotoCamera cam = new GPhotoCamera();
+
+		try {
+			int num_cameras = cam.DetectCameras();
+			int selected_cam;
+
+			if (num_cameras < 1) {
+				HigMessageDialog md = new HigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
+					MessageType.Warning, ButtonsType.Ok, 
+					Mono.Posix.Catalog.GetString ("No cameras detected."),
+					Mono.Posix.Catalog.GetString ("F-Spot was unable to find any cameras attached to this system." + 
+								      "  Double check that the camera is connected and has power")); 
+
+				md.Run ();
+				md.Destroy ();
+				return;
+			} else if (num_cameras == 1) {
+				selected_cam = 0;
+			} else {
+				FSpot.CameraSelectionDialog camselect = new FSpot.CameraSelectionDialog (cam.CameraList);
+				selected_cam = camselect.Run ();
+			}
+
+			if (selected_cam >= 0) {
+				cam.SelectCamera (selected_cam);	
+				cam.InitializeCamera ();
+
+				FSpot.CameraFileSelectionDialog selector = new FSpot.CameraFileSelectionDialog (cam, db);
+				selector.Run ();
+
+				UpdateQuery ();
+			}
+		}
+		catch (GPhotoException ge) {
+			System.Console.WriteLine (ge.ToString ());
+			HigMessageDialog md = new HigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
+				MessageType.Error, ButtonsType.Ok, 
+				Mono.Posix.Catalog.GetString ("Error connecting to camera"),
+				String.Format (Mono.Posix.Catalog.GetString ("Received error \"{0}\" while connecting to camera"), 
+				ge.Message));
+
+			md.Run ();
+			md.Destroy ();
+		} finally {
+			cam.ReleaseGPhotoResources ();
+		}
+	}
+	
 	unsafe void HandlePrintCommand (object sender, EventArgs e)
 	{
 		new FSpot.PrintDialog (SelectedPhotos ());
@@ -843,7 +915,7 @@ public class MainWindow {
 
 	void HandleExportToOriginal (object sender, EventArgs args)
 	{
-		new FSpot.OriginalExport (new FSpot.PhotoArray (SelectedPhotos ()));
+		new FSpot.FolderExport (new FSpot.PhotoArray (SelectedPhotos ()));
 	}
 
 	void HandleViewDirectory (object sender, EventArgs args)
@@ -904,7 +976,7 @@ public class MainWindow {
 			"Alessandro Gervaso",
 			"Peter Johanson",
 			"Grahm Orr",
-			"Cheslack-Postava",
+			"Ewen Cheslack-Postava",
 			"Patanjali Somayaji",
 			"Matt Jones",
 			"Martin Willemoes Hansen",
@@ -994,13 +1066,14 @@ public class MainWindow {
 	public void HandleCreateNewTagCommand (object sender, EventArgs args)
 	{
 		TagCommands.Create command = new TagCommands.Create (db.Tags, main_window);
-		command.Execute (TagCommands.TagType.Tag);
+		command.Execute (TagCommands.TagType.Tag, tag_selection_widget.TagHighlight ());
+		
 	}
 
 	public void HandleCreateNewCategoryCommand (object sender, EventArgs args)
 	{
 		TagCommands.Create command = new TagCommands.Create (db.Tags, main_window);
-		command.Execute (TagCommands.TagType.Category);
+		command.Execute (TagCommands.TagType.Category, tag_selection_widget.TagHighlight ());
 	}
 
 	public void HandleAttachTagCommand (object obj, EventArgs args)
@@ -1139,6 +1212,14 @@ public class MainWindow {
 			group_selector.Show ();
 	}
 
+	void HandleDisplayInfoSidebar (object sender, EventArgs args)
+	{
+		if (info_vpaned.Visible)
+			info_vpaned.Hide ();
+		else
+			info_vpaned.Show ();
+	}
+
 	void HandleViewSlideShow (object sender, EventArgs args)
 	{
 		main_window.GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.Watch);
@@ -1355,14 +1436,15 @@ public class MainWindow {
 	public void HandleRemoveCommand (object sender, EventArgs args)
 	{
    		Photo[] photos = SelectedPhotos();
-   		string header = Mono.Posix.Catalog.GetPluralString ("Remove the selected photo from the catalog?",
-								 "Remove the {0} selected photos from the catalog?", 
+   		string header = Mono.Posix.Catalog.GetPluralString ("Remove the selected photo from F-Spot?",
+								 "Remove the {0} selected photos from F-Spot?", 
 								 photos.Length);
 
 		header = String.Format (header, photos.Length);
-		string msg = Mono.Posix.Catalog.GetString("If you remove photos from the catalog all tag information will be lost. The photos remain on your computer and can be imported into f-spot again.");
+		string msg = Mono.Posix.Catalog.GetString("If you remove photos from the F-Spot catalog all tag information will be lost. The photos remain on your computer and can be imported into F-Spot again.");
 		string ok_caption = Mono.Posix.Catalog.GetString("_Remove from Catalog");
-		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(main_window, DialogFlags.DestroyWithParent, MessageType.Warning, header, msg, ok_caption)) {                              
+		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(main_window, DialogFlags.DestroyWithParent, 
+									   MessageType.Warning, header, msg, ok_caption)) {                              
 			foreach (Photo photo in photos) {
 				db.Photos.Remove (photo);
 			}
@@ -1557,6 +1639,8 @@ public class MainWindow {
 		copy.Sensitive = active_selection;
 
 		delete_selected_tag.Sensitive = tag_sensitive;
+		edit_selected_tag.Sensitive = tag_sensitive;
+
 		attach_tag_to_selection.Sensitive = tag_sensitive && active_selection;
 		remove_tag_from_selection.Sensitive = tag_sensitive && active_selection;
 	}
