@@ -53,6 +53,10 @@ public class PhotoView : EventBox {
 	private Label count_label;
 	private Entry description_entry;
 
+	private Gtk.Button crop_button;
+	private Gtk.Button redeye_button;
+	private Gtk.Button color_button;
+
 	FSpot.AsyncPixbufLoader loader = new FSpot.AsyncPixbufLoader ();
 
 	private const double MAX_ZOOM = 5.0;
@@ -124,11 +128,16 @@ public class PhotoView : EventBox {
 
 	private void UpdateButtonSensitivity ()
 	{
-		bool prev = CurrentPhotoValid () && CurrentPhoto > 0;
-		bool next = CurrentPhotoValid () && CurrentPhoto < query.Photos.Length -1;
+		bool valid = CurrentPhotoValid ();
+		bool prev = valid && CurrentPhoto > 0;
+		bool next = valid && CurrentPhoto < query.Photos.Length -1;
 
 		display_previous_button.Sensitive = prev;
 		display_next_button.Sensitive = next;
+
+		crop_button.Sensitive = valid;
+		redeye_button.Sensitive = valid;
+		color_button.Sensitive = valid;
 	}
 
 	private void UpdateCountLabel ()
@@ -208,42 +217,67 @@ public class PhotoView : EventBox {
 		DisplayPrevious ();
 	}
 
+
+	private void HandleRedEyeButtonClicked (object sender, EventArgs args)
+	{
+		ProcessImage (true);
+	}
+	
 	private void HandleCropButtonClicked (object sender, EventArgs args)
+	{
+		ProcessImage (false);
+	}
+
+	// FIXME this design sucks, I'm just doing it this way while
+	// I redesign the editing system.
+	private void ProcessImage (bool redeye)
 	{
 		int x, y, width, height;
 		if (! photo_view.GetSelection (out x, out y, out width, out height))
 			return;
 		
-		Pixbuf original_pixbuf = photo_view.Pixbuf;
+		Pixbuf original_pixbuf = photo_view.CompletePixbuf ();
 		if (original_pixbuf == null) {
 			Console.WriteLine ("No image");
 			return;
 		}
 
 		Photo photo = query.Photos [CurrentPhoto];
+		Exif.ExifData exif_data = new Exif.ExifData (photo.DefaultVersionPath);
 
-		Pixbuf cropped_pixbuf = new Pixbuf (original_pixbuf.Colorspace, false, original_pixbuf.BitsPerSample,
-						    width, height);
+		Pixbuf edited;
+		if (redeye) {
+			Gdk.Rectangle area = new Gdk.Rectangle (x, y, width, height);
+			edited = PixbufUtils.RemoveRedeye (original_pixbuf, 
+							   area);
+
+
+		} else { // Crop (I told you it was ugly)
+			edited = new Pixbuf (original_pixbuf.Colorspace, 
+					     false, original_pixbuf.BitsPerSample,
+					     width, height);
+			
+			original_pixbuf.CopyArea (x, y, width, height, edited, 0, 0);
+		}
 
-		original_pixbuf.CopyArea (x, y, width, height, cropped_pixbuf, 0, 0);		
 
 		// FIXME the fact that the selection doesn't go away is a bug in ImageView, it should
 		// be fixed there.
-		photo_view.Pixbuf = cropped_pixbuf;
+		photo_view.Pixbuf = edited;
 		photo_view.UnsetSelection ();
 
-		System.Console.WriteLine ("Got here");
-
 		try {
 			if (photo.DefaultVersionId == Photo.OriginalVersionId) {
 				photo.DefaultVersionId = photo.CreateDefaultModifiedVersion (photo.DefaultVersionId, false);
-				cropped_pixbuf.Savev (photo.DefaultVersionPath, "jpeg", null, null);
-				PhotoStore.GenerateThumbnail (photo.DefaultVersionPath);
+				PixbufUtils.SaveJpeg (edited, photo.DefaultVersionPath, 
+						      95, exif_data);
+				FSpot.ThumbnailGenerator.Create (photo.DefaultVersionPath).Dispose ();
 				query.Commit (CurrentPhoto);
 			} else {
 				// FIXME we need to invalidate the thumbnail in the cache as well
-				cropped_pixbuf.Savev (photo.DefaultVersionPath, "jpeg", null, null);
-				PhotoStore.GenerateThumbnail (photo.DefaultVersionPath);
+				PixbufUtils.SaveJpeg (edited, photo.DefaultVersionPath, 
+						      95, exif_data);
+				FSpot.ThumbnailGenerator.Create (photo.DefaultVersionPath).Dispose ();
 				query.MarkChanged (CurrentPhoto);
 			}
 		} catch (GLib.GException ex) {
@@ -257,7 +291,7 @@ public class PhotoView : EventBox {
 			PhotoChanged (this);
 	}
 
-	private void HandleUnsharpButtonClicked (object sender, EventArgs args) {
+	private void HandleColorButtonClicked (object sender, EventArgs args) {
 		new FSpot.ColorDialog (photo_view);
 	}	
 
@@ -338,13 +372,11 @@ public class PhotoView : EventBox {
 	private void HandlePhotoChanged (FSpot.PhotoImageView view)
 	{
 		CommitPendingChanges ();
+		bool valid = CurrentPhotoValid ();
 
 		Update ();
 
-		if (CurrentPhotoValid ()) 
-			tag_view.Current = query.Photos [CurrentPhoto];
-		else 
-			tag_view.Current = null;
+		tag_view.Current = valid ? query.Photos [CurrentPhoto] : null;
 
 		if (this.PhotoChanged != null)
 			PhotoChanged (this);
@@ -412,19 +444,23 @@ public class PhotoView : EventBox {
 
 		toolbar_hbox.PackStart (CreateConstraintsOptionMenu (), false, false, 0);
 
-		Button crop_button = new ToolbarButton ();
-		Gtk.Image crop_button_icon = new Gtk.Image ("f-spot-crop", IconSize.Button);
-		crop_button.Add (crop_button_icon);
+		crop_button = new ToolbarButton ();
+		crop_button.Add (new Gtk.Image ("f-spot-crop", IconSize.Button));
 		toolbar_hbox.PackStart (crop_button, false, true, 0);
 	
 		crop_button.Clicked += new EventHandler (HandleCropButtonClicked);
 
-		Button unsharp_button = new ToolbarButton ();
-		Gtk.Image unsharp_button_icon = new Gtk.Image ("f-spot-edit-image", IconSize.Button);
-		unsharp_button.Add (unsharp_button_icon);
-		toolbar_hbox.PackStart (unsharp_button, false, true, 0);
+		redeye_button = new ToolbarButton ();
+		redeye_button.Add (new Gtk.Image ("f-spot-red-eye", IconSize.Button));
+		toolbar_hbox.PackStart (redeye_button, false, true, 0);
 	
-		unsharp_button.Clicked += new EventHandler (HandleUnsharpButtonClicked);
+		redeye_button.Clicked += new EventHandler (HandleRedEyeButtonClicked);
+
+		color_button = new ToolbarButton ();
+		color_button.Add (new Gtk.Image ("f-spot-edit-image", IconSize.Button));
+		toolbar_hbox.PackStart (color_button, false, true, 0);
+	
+		color_button.Clicked += new EventHandler (HandleColorButtonClicked);
 
 		/* Spacer Label */
 		toolbar_hbox.PackStart (new Label (""), true, true, 0);
